@@ -26,8 +26,8 @@
 #include <KPublicTransport/Location>
 #include <KPublicTransport/Manager>
 #include <KPublicTransport/OnboardStatus>
-#include <KPublicTransport/StopoverReply>
-#include <KPublicTransport/StopoverRequest>
+#include <KPublicTransport/TripReply>
+#include <KPublicTransport/TripRequest>
 
 #include <KPkPass/Pass>
 
@@ -177,6 +177,14 @@ void LiveDataManager::checkForUpdates()
     pollForUpdates(true);
 }
 
+void LiveDataManager::checkForUpdates(const QStringList &batchIds)
+{
+    clearArrived();
+    for (const auto &batchId : batchIds) {
+        pollBatchForUpdates(batchId, true);
+    }
+}
+
 [[nodiscard]] static bool canSelectBackend(const KPublicTransport::Location &loc)
 {
     return loc.hasCoordinate() || loc.country().size() == 2;
@@ -185,122 +193,45 @@ void LiveDataManager::checkForUpdates()
 void LiveDataManager::checkReservation(const QVariant &res, const QString &resId)
 {
     using namespace KPublicTransport;
-    const auto arrived = hasArrived(resId, res);
-
-    // load full journey if we don't have one yet
-    if (!arrived && data(resId).journey.mode() == JourneySection::Invalid) {
-        const auto from = PublicTransport::locationFromPlace(LocationUtil::departureLocation(res), res);
-        const auto to = PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res), res);
-        JourneyRequest req(from, to);
-        // start searching slightly earlier, so leading walking section because our coordinates
-        // aren't exactly at the right spot wont make the routing service consider the train we
-        // are looking for as impossible to reach on time
-        req.setDateTime(SortUtil::startDateTime(res).addSecs(-600));
-        req.setDateTimeMode(JourneyRequest::Departure);
-        req.setIncludeIntermediateStops(true);
-        req.setIncludePaths(true);
-        req.setModes(JourneySection::PublicTransport);
-        PublicTransport::selectBackends(req, m_ptMgr, res);
-        if (canSelectBackend(from) || canSelectBackend(to) || !req.backendIds().isEmpty()) {
-            auto reply = m_ptMgr->queryJourney(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() {
-                journeyQueryFinished(reply, resId);
-            });
-            m_lastPollAttempt.insert(resId, now());
-            return;
-        }
-    }
-
-    if (!hasDeparted(resId, res)) {
-        StopoverRequest req(PublicTransport::locationFromPlace(LocationUtil::departureLocation(res), res));
-        req.setMode(StopoverRequest::QueryDeparture);
-        req.setDateTime(SortUtil::startDateTime(res));
-        PublicTransport::selectBackends(req, m_ptMgr, res);
-        if (canSelectBackend(req.stop()) || !req.backendIds().isEmpty()) {
-            auto reply = m_ptMgr->queryStopover(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() {
-                stopoverQueryFinished(reply, LiveData::Departure, resId);
-            });
-            m_lastPollAttempt.insert(resId, now());
-        }
-    }
-
-    if (!arrived) {
-        StopoverRequest req(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res), res));
-        req.setMode(StopoverRequest::QueryArrival);
-        req.setDateTime(SortUtil::endDateTime(res));
-        PublicTransport::selectBackends(req, m_ptMgr, res);
-        if (canSelectBackend(req.stop()) || !req.backendIds().isEmpty()) {
-            auto reply = m_ptMgr->queryStopover(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() {
-                stopoverQueryFinished(reply, LiveData::Arrival, resId);
-            });
-            m_lastPollAttempt.insert(resId, now());
-        }
-    }
-}
-
-void LiveDataManager::stopoverQueryFinished(KPublicTransport::StopoverReply *reply, LiveData::Type type, const QString &resId)
-{
-    reply->deleteLater();
-    if (reply->error() != KPublicTransport::Reply::NoError) {
-        qCDebug(Log) << reply->error() << reply->errorString();
-        return;
-    }
-    stopoverQueryFinished(reply->takeResult(), type, resId);
-}
-
-void LiveDataManager::stopoverQueryFinished(std::vector<KPublicTransport::Stopover> &&result, LiveData::Type type, const QString &resId)
-{
-    const auto res = m_resMgr->reservation(resId);
-    for (const auto &stop : result) {
-        qCDebug(Log) << "Got stopover information:" << stop.route().line().name() << stop.scheduledDepartureTime();
-        if (type == LiveData::Arrival ? PublicTransportMatcher::isArrivalForReservation(res, stop)
-                                      : PublicTransportMatcher::isDepartureForReservation(res, stop)) {
-            qCDebug(Log) << "Found stopover information:" << stop.route().line().name() << stop.expectedPlatform() << stop.expectedDepartureTime();
-            updateStopoverData(stop, type, resId, res);
-            return;
-        }
-    }
-
-    // record this is a failed lookup so we don't try again
-    data(resId).setTimestamp(type, now());
-}
-
-void LiveDataManager::journeyQueryFinished(KPublicTransport::JourneyReply *reply, const QString &resId)
-{
-    reply->deleteLater();
-    if (reply->error() != KPublicTransport::Reply::NoError) {
-        qCDebug(Log) << reply->error() << reply->errorString();
+    if (hasArrived(resId, res)) {
         return;
     }
 
-    using namespace KPublicTransport;
-    const auto res = m_resMgr->reservation(resId);
-    for (const auto &journey : reply->result()) {
-        if (std::count_if(journey.sections().begin(),
-                          journey.sections().end(),
-                          [](const auto &sec) {
-                              return sec.mode() == JourneySection::PublicTransport;
-                          })
-            != 1) {
-            continue;
-        }
-        const auto it = std::find_if(journey.sections().begin(), journey.sections().end(), [](const auto &sec) {
-            return sec.mode() == JourneySection::PublicTransport;
+    auto jny = data(resId).journey;
+    if (jny.mode() == JourneySection::Invalid) {
+        jny.setMode(JourneySection::PublicTransport);
+        jny.setFrom(PublicTransport::locationFromPlace(LocationUtil::departureLocation(res), res));
+        jny.setTo(PublicTransport::locationFromPlace(LocationUtil::arrivalLocation(res), res));
+        jny.setScheduledDepartureTime(SortUtil::startDateTime(res));
+        jny.setRoute(PublicTransport::routeForReservation(res));
+    }
+
+    TripRequest req(jny);
+    req.setDownloadAssets(true);
+    PublicTransport::selectBackends(req, m_ptMgr, res);
+    if (canSelectBackend(jny.from()) || canSelectBackend(jny.to()) || !req.backendIds().isEmpty() || jny.hasIdentifiers()) {
+        auto reply = m_ptMgr->queryTrip(req);
+        connect(reply, &Reply::finished, this, [this, resId, reply]() {
+            reply->deleteLater();
+            if (reply->error() == Reply::NoError) {
+                auto jny = reply->journeySection();
+                if (jny.mode() == JourneySection::PublicTransport) {
+                    applyJourney(resId, jny);
+                    return;
+                }
+            }
+            // record this is a failed lookup so we don't try again
+            tripQueryFailed(resId);
         });
-        assert(it != journey.sections().end());
-        qCDebug(Log) << "Got journey information:" << (*it).route().line().name() << (*it).scheduledDepartureTime();
-        if (PublicTransportMatcher::isJourneyForReservation(res, (*it))) {
-            qCDebug(Log) << "Found journey information:" << (*it).route().line().name() << (*it).expectedDeparturePlatform() << (*it).expectedDepartureTime();
-            updateJourneyData((*it), resId, res);
-            return;
-        }
+        m_lastPollAttempt.insert(resId, now());
     }
+}
 
-    // record this is a failed lookup so we don't try again
+void LiveDataManager::tripQueryFailed(const QString &resId)
+{
     data(resId).setTimestamp(LiveData::Arrival, now());
     data(resId).setTimestamp(LiveData::Departure, now());
+    data(resId).setTimestamp(LiveData::Journey, now());
 }
 
 static KPublicTransport::Stopover applyLayoutData(const KPublicTransport::Stopover &stop, const KPublicTransport::Stopover &layout)
@@ -330,45 +261,6 @@ static void applyMissingStopoverData(KPublicTransport::Stopover &stop, const KPu
     }
 }
 
-void LiveDataManager::updateStopoverData(const KPublicTransport::Stopover &stop, LiveData::Type type, const QString &resId, const QVariant &res)
-{
-    auto &ld = data(resId);
-    const auto oldStop = ld.stopover(type);
-    auto newStop = stop;
-    newStop.applyMetaData(true); // download logo assets if needed
-
-    // retain already existing vehicle/platform layout data if we are still departing/arriving in the same place
-    if (PublicTransport::isSameStopoverForLayout(newStop, oldStop)) {
-        newStop = applyLayoutData(newStop, oldStop);
-    }
-    applyMissingStopoverData(newStop, oldStop);
-
-    ld.setStopover(type, newStop);
-    ld.setTimestamp(type, now());
-    if (type == LiveData::Departure) {
-        ld.journey.setDeparture(newStop);
-    } else {
-        ld.journey.setArrival(newStop);
-    }
-    ld.journeyTimestamp = now();
-    ld.store(resId);
-
-    // update reservation with live data
-    const auto newRes = type == LiveData::Arrival ? PublicTransport::mergeArrival(res, newStop) : PublicTransport::mergeDeparture(res, newStop);
-    if (!ReservationHelper::equals(res, newRes)) {
-        m_resMgr->updateReservation(resId, newRes);
-    }
-
-    // emit update signals
-    Q_EMIT type == LiveData::Arrival ? arrivalUpdated(resId) : departureUpdated(resId);
-    Q_EMIT journeyUpdated(resId);
-
-    // check if we need to notify
-    if (NotificationHelper::shouldNotify(oldStop, newStop, type)) {
-        showNotification(resId, ld);
-    }
-}
-
 static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, const KPublicTransport::JourneySection &oldJny)
 {
     if (journey.intermediateStops().size() != oldJny.intermediateStops().size()) {
@@ -407,8 +299,8 @@ static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, c
 void LiveDataManager::updateJourneyData(const KPublicTransport::JourneySection &journey, const QString &resId, const QVariant &res)
 {
     auto &ld = data(resId);
-    const auto oldDep = ld.stopover(LiveData::Departure);
-    const auto oldArr = ld.stopover(LiveData::Arrival);
+    const auto oldDep = ld.departure;
+    const auto oldArr = ld.arrival;
     const auto oldJny = ld.journey;
     ld.journey = journey;
 
@@ -655,6 +547,43 @@ void LiveDataManager::poll()
 }
 
 void LiveDataManager::pollForUpdates(bool force)
+{
+    clearArrived();
+
+    for (const auto &batchId : m_reservations) {
+        pollBatchForUpdates(batchId, force);
+    }
+}
+
+void LiveDataManager::pollBatchForUpdates(const QString &batchId, bool force)
+{
+    if (!force && nextPollTimeForReservation(batchId) > 60 * 1000) {
+        // data is still "fresh" according to the poll policy
+        return;
+    }
+
+    const auto res = m_resMgr->reservation(batchId);
+    if (hasArrived(batchId, res)) {
+        return;
+    }
+
+    if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
+        checkReservation(res, batchId);
+    }
+
+    // check for pkpass updates, for each element in this batch
+    const auto resIds = m_resMgr->reservationsForBatch(batchId);
+    for (const auto &resId : resIds) {
+        const auto res = m_resMgr->reservation(resId);
+        const auto passId = PkPassManager::passId(res);
+        if (!passId.isEmpty()) {
+            m_lastPollAttempt.insert(batchId, now());
+            m_pkPassMgr->updatePass(passId);
+        }
+    }
+}
+
+void LiveDataManager::clearArrived()
 {
     for (auto it = m_reservations.begin(); it != m_reservations.end();) {
         const auto batchId = *it;
@@ -665,28 +594,8 @@ void LiveDataManager::pollForUpdates(bool force)
             cancelNotification(*it);
             it = m_reservations.erase(it);
             m_lastPollAttempt.remove(batchId);
-            continue;
-        }
-        ++it;
-
-        if (!force && nextPollTimeForReservation(batchId) > 60 * 1000) {
-            // data is still "fresh" according to the poll policy
-            continue;
-        }
-
-        if (JsonLd::isA<TrainReservation>(res) || JsonLd::isA<BusReservation>(res)) {
-            checkReservation(res, batchId);
-        }
-
-        // check for pkpass updates, for each element in this batch
-        const auto resIds = m_resMgr->reservationsForBatch(batchId);
-        for (const auto &resId : resIds) {
-            const auto res = m_resMgr->reservation(resId);
-            const auto passId = m_pkPassMgr->passId(res);
-            if (!passId.isEmpty()) {
-                m_lastPollAttempt.insert(batchId, now());
-                m_pkPassMgr->updatePass(passId);
-            }
+        } else {
+            ++it;
         }
     }
 }
