@@ -12,6 +12,7 @@
 #include <KItinerary/BusTrip>
 #include <KItinerary/Event>
 #include <KItinerary/ExtractorPostprocessor>
+#include <KItinerary/Flight>
 #include <KItinerary/Reservation>
 #include <KItinerary/Ticket>
 #include <KItinerary/TrainTrip>
@@ -29,6 +30,8 @@
 #include <QDebug>
 #include <QUrl>
 
+using namespace Qt::Literals;
+
 bool PublicTransport::isBusMode(KPublicTransport::Line::Mode mode)
 {
     using namespace KPublicTransport;
@@ -110,49 +113,6 @@ QVariant PublicTransport::busStationFromLocation(const KPublicTransport::Locatio
     return PublicTransport::placeFromLocation<KItinerary::BusStation>(loc);
 }
 
-QString PublicTransport::journeySectionLabel(const KPublicTransport::JourneySection &journeySection) const
-{
-    using namespace KPublicTransport;
-    switch (journeySection.mode()) {
-    case JourneySection::Invalid:
-        break;
-    case JourneySection::PublicTransport:
-        return journeySection.route().line().name();
-    case JourneySection::Walking:
-        return i18n("Walk");
-    case JourneySection::Waiting:
-        return i18n("Wait");
-    case JourneySection::Transfer:
-        break; // ?
-    case JourneySection::RentedVehicle:
-        switch (journeySection.rentalVehicle().type()) {
-        case RentalVehicle::Unknown:
-            break;
-        case RentalVehicle::Bicycle:
-        case RentalVehicle::Pedelec:
-            return i18n("Rented bicycle");
-        case RentalVehicle::ElectricMoped:
-            return i18n("Rented moped");
-        case RentalVehicle::ElectricKickScooter:
-            return i18n("Rented kick scooter");
-        case RentalVehicle::Car:
-            return i18n("Rented car");
-        }
-        break;
-    case JourneySection::IndividualTransport:
-        switch (journeySection.individualTransport().mode()) {
-        case IndividualTransport::Bike:
-            return i18n("Bicycle");
-        case IndividualTransport::Car:
-            return i18n("Car");
-        case IndividualTransport::Walk:
-            return i18n("Walk");
-        }
-        break;
-    }
-    return {};
-}
-
 static KItinerary::Ticket clearSeat(KItinerary::Ticket ticket)
 {
     auto seat = ticket.ticketedSeat();
@@ -351,50 +311,6 @@ QString PublicTransport::attributionSummary(const QVariantList &attributions) co
     return QLocale().createSeparatedList(l);
 }
 
-bool PublicTransport::warnAboutSection(const KPublicTransport::JourneySection &section) const
-{
-    using namespace KPublicTransport;
-
-    switch (section.mode()) {
-    case JourneySection::Invalid:
-        return false;
-    case JourneySection::Walking:
-    case JourneySection::Transfer:
-        return section.duration() > 20 * 60 || section.distance() > 1000;
-    case JourneySection::Waiting:
-        return section.duration() > 20 * 60;
-    case JourneySection::PublicTransport:
-        return std::any_of(section.loadInformation().begin(), section.loadInformation().end(), [](const auto &load) {
-            return load.load() == Load::Full;
-        });
-    case JourneySection::RentedVehicle:
-        switch (section.rentalVehicle().type()) {
-        case RentalVehicle::Unknown:
-            return false;
-        case RentalVehicle::Bicycle:
-            return section.duration() > 60 * 60 || section.distance() > 20000;
-        case RentalVehicle::ElectricKickScooter:
-        case RentalVehicle::ElectricMoped:
-        case RentalVehicle::Pedelec:
-        case RentalVehicle::Car:
-            return false; // ???
-        }
-        break;
-    case JourneySection::IndividualTransport:
-        switch (section.individualTransport().mode()) {
-        case IndividualTransport::Walk:
-            return section.duration() > 20 * 60 || section.distance() > 1000;
-        case IndividualTransport::Bike:
-            return section.duration() > 60 * 60 || section.distance() > 20000;
-        case IndividualTransport::Car:
-            return false;
-        }
-        break;
-    }
-
-    return false;
-}
-
 QString PublicTransport::idenfitierFromLocation(const KPublicTransport::Location &loc)
 {
     auto id = loc.identifier(QLatin1StringView("ibnr"));
@@ -528,14 +444,36 @@ bool PublicTransport::isSameStopoverForLayout(const KPublicTransport::Stopover &
     return lhsPlatform == rhsPlatform;
 }
 
-KPublicTransport::Load::Category PublicTransport::maximumOccupancy(const QList<KPublicTransport::LoadInfo> &loadInfo)
+KPublicTransport::Route PublicTransport::routeForReservation(const QVariant &res)
 {
+    using namespace KItinerary;
     using namespace KPublicTransport;
-    auto o = Load::Unknown;
-    for (const auto &l : loadInfo) {
-        o = std::max(o, l.load());
+
+    Line line;
+    if (JsonLd::isA<TrainReservation>(res)) {
+        const auto train = res.value<TrainReservation>().reservationFor().value<TrainTrip>();
+        line.setMode(Line::Train);
+        line.setName(QString(train.trainName() + ' '_L1 + train.trainNumber()).trimmed());
+    } else if (JsonLd::isA<BusReservation>(res)) {
+        const auto bus = res.value<BusReservation>().reservationFor().value<BusTrip>();
+        line.setMode(Line::Bus);
+        line.setName(QString(bus.busName() + ' '_L1 + bus.busNumber()).trimmed());
+    } else if (JsonLd::isA<FlightReservation>(res)) {
+        const auto flight = res.value<FlightReservation>().reservationFor().value<Flight>();
+        line.setMode(Line::Air);
+        line.setName(flight.airline().iataCode() + ' '_L1 + flight.flightNumber());
+    } else if (JsonLd::isA<BoatReservation>(res)) {
+        const auto boat = res.value<BoatReservation>().reservationFor().value<BoatTrip>();
+        line.setMode(Line::Boat);
+        line.setName(boat.name());
+    } else {
+        qCDebug(Log) << "Unhandled reservation type" << res;
+        return {};
     }
-    return o;
+
+    Route route;
+    route.setLine(line);
+    return route;
 }
 
 #include "moc_publictransport.cpp"
